#include <LiquidCrystal_I2C.h>
#include <Keypad.h> 
#include <Servo.h>
#include <EEPROM.h>

#define UNLOCK_POS 1
#define LOCK_POS 99

LiquidCrystal_I2C lcd(0x27, 16, 2);
Servo myservo;

int buzzer = PIN_PC0;
int ledR = PIN_PC2;
int ledG = PIN_PC3;
char fctpass[7];
char fpassword[] = {48, 48, 48, 48};
char password[4];
char initial_password[4], new_password[4];
int i = 0;
char key_pressed = 0;
bool changepass = 0;
bool factory = 0;

const byte rows = 4;
const byte columns = 3;

char hexaKeys[rows][columns] = {
  {'1', '2', '3',},
  {'4', '5', '6',},
  {'7', '8', '9',},
  {'*', '0', '#',}
};

byte row_pins[rows] = {PIN_PD0, PIN_PB5, PIN_PB4, PIN_PB3};
byte column_pins[columns] = {PIN_PB2, PIN_PB1, PIN_PB0};

Keypad keypad_key = Keypad( makeKeymap(hexaKeys), row_pins, column_pins, rows, columns);

void setup() {
  Serial.begin(9600);
  myservo.attach(PIN_PC1);
  pinMode(buzzer, OUTPUT);
  pinMode(ledR,OUTPUT);
  pinMode(ledG,OUTPUT);

  lcd.init();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("  - Welcome -");
  lcd.setCursor(0, 1);
  lcd.print(" Practicum Safe");
  delay(5000);
  lcd.clear();
  lcd.print("Enter Password");
  lcd.setCursor(0, 1);

  changepass = EEPROM.read(10);
  if (changepass == 1) {
    for (int j = 0; j < 4; j++)
      initial_password[j] = EEPROM.read(j);
  } else {
    initialpassword();
    factoryWrite();
  }

  factoryRead();
}

void loop() {
  lock();
  key_pressed = keypad_key.getKey();

  if (key_pressed == '#')
    change();

  if (factory == 1)
    changefct();

  if (key_pressed) {
    password[i++] = key_pressed;
    lcd.setCursor(i, 1);
    lcd.print("*");
  }

  if (factory == 0) {

    if (i == 4) {
      delay(200);
      for (int j = 0; j < 4; j++)
        initial_password[j] = EEPROM.read(j);
      if (!(strncmp(password, initial_password, 4)))
      {
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Correct!");
        digitalWrite(ledG,1);
        unlock();
        delay(30000);
        digitalWrite(ledG,0);
        lcd.setCursor(0, 1);
        lcd.print("Use # to change");
        delay(2000);
        lcd.clear();
        lcd.print("Enter Password");
        lcd.setCursor(0, 1);
        i = 0;
      }
      else {
        if (!(strncmp(password, fpassword, 4))) {
          factory = 1;
        }
        
        lock();
        lcd.clear();
        lcd.print("Arkkkkkk");
        lcd.setCursor(0, 1);
        lcd.print("Use # to change");
        tone(buzzer, 400);
        digitalWrite(ledR,1);
        delay(5000);
        noTone(buzzer);
        digitalWrite(ledR,0);
        delay(2000);
        lcd.clear();
        lcd.print("Enter Password");
        lcd.setCursor(0, 1);
        i = 0;

      }
    }
  }
}


void changefct() { //6110503
  int j = 0;
  lcd.clear();
  lcd.print("factory Password");
  lcd.setCursor(0, 1);
  while (j < 7) {
    char key = keypad_key.getKey();
    if (key) {
      new_password[j++] = key;
      lcd.print("*");
    }
    key = 0;
  }
  delay(500);
  if ((strncmp(new_password, fctpass, 7))) {
    lcd.clear();
    lcd.print("Arkkkkkk");
    lcd.setCursor(0, 1);
    lcd.print("Try Again");
    tone(buzzer, 400);
    digitalWrite(ledR,1);
    delay(5000);
    noTone(buzzer);
    digitalWrite(ledR,0);
    delay(1000);
  }
  else {
    j = 0;
    lcd.clear();
    lcd.print("New Password");
    lcd.setCursor(0, 1);
    while (j < 4)    {
      char key = keypad_key.getKey();
      if (key) {
        initial_password[j] = key;
        lcd.print("*");
        EEPROM.write(j, key);
        j++;
      }
    }
    changepass = 1;
    EEPROM.write(10, changepass);
    lcd.print("    Changed");
    digitalWrite(ledG,1);
    delay(4000);
    digitalWrite(ledG,0);
    delay(1000);
    
  }
  lcd.clear();
  lcd.print("Enter Password");
  lcd.setCursor(0, 1);
  key_pressed = 0;
  factory = 0;
}


void change() {
  int j = 0;
  lcd.clear();
  lcd.print("Current Password");
  lcd.setCursor(0, 1);
  while (j < 4) {
    char key = keypad_key.getKey();
    if (key) {
      new_password[j++] = key;
      lcd.print("*");
    }
    key = 0;
  }
  delay(500);
  if ((strncmp(new_password, initial_password, 4))) {
    lcd.clear();
    lcd.print("Arkkkkkk");
    lcd.setCursor(0, 1);
    lcd.print("Try Again");
    digitalWrite(ledR,1);
    delay(4000);
    digitalWrite(ledR,0);
    delay(1000);
  }
  else {
    j = 0;
    lcd.clear();
    lcd.print("New Password");
    lcd.setCursor(0, 1);
    while (j < 4)    {
      char key = keypad_key.getKey();
      if (key) {
        initial_password[j] = key;
        lcd.print("*");
        EEPROM.write(j, key);
        j++;
      }
    }
    changepass = 1;
    EEPROM.write(10, changepass);
    lcd.print("    Changed");
    digitalWrite(ledG,1);
    delay(4000);
    digitalWrite(ledG,0);
    delay(1000);
  }
  lcd.clear();
  lcd.print("Enter Password");
  lcd.setCursor(0, 1);
  key_pressed = 0;
}

void initialpassword() {
  for (int j = 0; j < 4; j++)
    EEPROM.write(j, j + 49);
  for (int j = 0; j < 4; j++)
    initial_password[j] = EEPROM.read(j);
}

void factoryWrite() {  //6110503
  EEPROM.write(20, 54);
  EEPROM.write(21, 49);
  EEPROM.write(22, 49);
  EEPROM.write(23, 48);
  EEPROM.write(24, 53);
  EEPROM.write(25, 48);
  EEPROM.write(26, 51);
}

void factoryRead() {  //6110503
  fctpass[0] = EEPROM.read(20);
  fctpass[1] = EEPROM.read(21);
  fctpass[2] = EEPROM.read(22);
  fctpass[3] = EEPROM.read(23);
  fctpass[4] = EEPROM.read(24);
  fctpass[5] = EEPROM.read(25);
  fctpass[6] = EEPROM.read(26);
}

void lock() {
  myservo.write(LOCK_POS);
}

void unlock() {
  myservo.write(UNLOCK_POS);
}
